<?php
if (!defined('ABSPATH')) {
    exit;
}

/**
 * WC_Gateway_Ipasspay class.
 *
 * @extends WC_Payment_Gateway
 */
//class WC_Gateway_Ipasspay_Direct extends WC_Payment_Gateway_CC {
class WC_Gateway_Ipasspay_Direct extends WC_Payment_Gateway_CC {
    /**
     * Checkout Locale
     *
     * @var string
     */
    public $ipasspay_direct_checkout_locale;

    /**
     * Constructor
     */
    public function __construct() {
        $this->id = 'ipasspay-direct';
        $this->method_title = __('Ipasspay-Direct', 'woocommerce-gateway-ipasspay-direct');
        $this->method_description = __('Ipasspay works by adding credit card fields on the checkout and then sending the details to Ipasspay for verification.', 'woocommerce-gateway-ipasspay-direct');
        $this->has_fields = true;

        // Load the form fields.
        $this->init_form_fields();

        // Load the settings.
        $this->init_settings();

        // Get setting values.
        $this->title = $this->get_option('title');
        $this->description = $this->get_option('description');
        $this->enabled = $this->get_option('enabled');
        $this->ipasspay_direct_checkout_locale = $this->get_option('ipasspay_direct_checkout_locale');

        //Get config values
        $this->mid = $this->get_option('mid');
        $this->app_id = $this->get_option('app_id');
        $this->api_secret = $this->get_option('api_secret');
        $this->gateway_url = $this->get_option('gateway_url');
        $this->gateway_version = $this->get_option('gateway_version');
        $this->asyn_url = $this->get_option('asyn_url');

        // Hooks.
        //add_action('wp_enqueue_scripts', array($this, 'payment_scripts'));
        add_action('woocommerce_update_options_payment_gateways_' . $this->id, array($this, 'process_admin_options'));
        //添加一个自定义回调
        add_action( 'woocommerce_api_wc_gateway_ipasspay_direct', array($this,'notify'));
    }

    /**
     * Get_icon function.
     *
     * @access public
     * @return string
     */
    public function get_icon() {
        $ext = version_compare(WC()->version, '2.6', '>=') ? '.svg' : '.png';
        $style = version_compare(WC()->version, '2.6', '>=') ? 'style="margin-left: 0.3em"' : '';

        $icon = '<img src="' . WC_HTTPS::force_https_url(WC()->plugin_url() . '/assets/images/icons/credit-cards/visa' . $ext) . '" alt="Visa" width="32" ' . $style . ' />';
        $icon .= '<img src="' . WC_HTTPS::force_https_url(WC()->plugin_url() . '/assets/images/icons/credit-cards/mastercard' . $ext) . '" alt="Mastercard" width="32" ' . $style . ' />';
        $icon .= '<img src="' . WC_HTTPS::force_https_url(WC()->plugin_url() . '/assets/images/icons/credit-cards/amex' . $ext) . '" alt="Amex" width="32" ' . $style . ' />';
        $icon .= '<img src="' . WC_HTTPS::force_https_url(WC()->plugin_url() . '/assets/images/icons/credit-cards/jcb' . $ext) . '" alt="JCB" width="32" ' . $style . ' />';

        return apply_filters('woocommerce_gateway_icon', $icon, $this->id);
    }

    /**
     * Get Ipasspay-Direct amount to pay
     *
     * @param float  $total Amount due.
     * @param string $currency Accepted currency.
     *
     * @return float|int
     */
    public function get_ipasspay_direct_amount($total, $currency = '') {
        if (!$currency) {
            $currency = get_woocommerce_currency();
        }
        switch (strtoupper($currency)) {
            // Zero decimal currencies.
            case 'BIF':
            case 'CLP':
            case 'DJF':
            case 'GNF':
            case 'JPY':
            case 'KMF':
            case 'KRW':
            case 'MGA':
            case 'PYG':
            case 'RWF':
            case 'VND':
            case 'VUV':
            case 'XAF':
            case 'XOF':
            case 'XPF':
                $total = absint($total);
                break;
            default:
                $total = round($total, 2);
                break;
        }
        return $total;
    }

    /**
     * Check if this gateway is enabled
     */
    public function is_available() {
        if ('yes' === $this->enabled) {
            return true;
        }
        return false;
    }

    /**
     * Initialise Gateway Settings Form Fields
     */
    public function init_form_fields() {
        $this->form_fields = include 'settings-ipasspay-direct.php';
    }

    /**
     * Payment form on checkout page
     */
    public function payment_fields() {

        $user = wp_get_current_user();
        $display_tokenization = $this->supports('tokenization') && is_checkout() && $this->saved_cards;
        $total = WC()->cart->total;

        // If paying from order, we need to get total from order not cart.
        if (isset($_GET['pay_for_order']) && !empty($_GET['key'])) {
            $order = wc_get_order(wc_get_order_id_by_order_key(wc_clean($_GET['key'])));
            $total = $order->get_total();
        }

        if ($user->ID) {
            $user_email = get_user_meta($user->ID, 'billing_email', true);
            $user_email = $user_email ? $user_email : $user->user_email;
        } else {
            $user_email = '';
        }

        if (is_add_payment_method_page()) {
            $pay_button_text = __('Add Card', 'woocommerce-gateway-ipasspay-direct');
        } else {
            $pay_button_text = '';
        }

        echo '<div
				id="ipasspay-direct-payment-data"
				data-panel-label="' . esc_attr($pay_button_text) . '"
				data-description=""
				data-email="' . esc_attr($user_email) . '"
				data-amount="' . esc_attr($this->get_ipasspay_direct_amount(WC()->cart->total)) . '"
				data-name="' . esc_attr(get_bloginfo('name', 'display')) . '"
				data-currency="' . esc_attr(strtolower(get_woocommerce_currency())) . '"
				data-image="' . esc_attr($this->ipasspay_direct_checkout_image) . '"
				data-bitcoin="' . esc_attr($this->bitcoin ? 'true' : 'false') . '"
				data-locale="' . esc_attr($this->ipasspay_direct_checkout_locale ? $this->ipasspay_direct_checkout_locale : 'en') . '">';

        if ($this->description) {
            echo apply_filters('wc_ipasspay_direct_description', wpautop(wp_kses_post($this->description)));
        }
        $this->form();

        echo '</div>';

    }

    /**
     * Localize Ipasspay-Direct messages based on code
     *
     * @since 1.0.1
     * @version 1.0.1
     * @return array
     */
    public function get_localized_messages() {
        return apply_filters('wc_ipasspay__localized_messages', array(
            'invalid_number' => __('The card number is not a valid credit card number.', 'woocommerce-gateway-ipasspay-direct'),
            'invalid_expiry_month' => __('The card\'s expiration month is invalid.', 'woocommerce-gateway-ipasspay-direct'),
            'invalid_expiry_year' => __('The card\'s expiration year is invalid.', 'woocommerce-gateway-ipasspay-direct'),
            'invalid_cvc' => __('The card\'s security code is invalid.', 'woocommerce-gateway-ipasspay-direct'),
            'incorrect_number' => __('The card number is incorrect.', 'woocommerce-gateway-ipasspay-direct'),
            'expired_card' => __('The card has expired.', 'woocommerce-gateway-ipasspay-direct'),
            'incorrect_cvc' => __('The card\'s security code is incorrect.', 'woocommerce-gateway-ipasspay-direct'),
            'incorrect_zip' => __('The card\'s zip code failed validation.', 'woocommerce-gateway-ipasspay-direct'),
            'card_declined' => __('The card was declined.', 'woocommerce-gateway-ipasspay-direct'),
            'missing' => __('There is no card on a customer that is being charged.', 'woocommerce-gateway-ipasspay-direct'),
            'processing_error' => __('An error occurred while processing the card.', 'woocommerce-gateway-ipasspay-direct'),
            'invalid_request_error' => __('Could not find payment information.', 'woocommerce-gateway-ipasspay-direct'),
        ));
    }

    //删除空格
    private function trimall($str) {
        $qian = array(" ", "　", "\t", "\n", "\r");
        $hou = array("", "", "", "", "");
        return str_replace($qian, $hou, $str);
    }

    /**
     * the function for creating uuid in PHP
     */
    private function create_guid() {
        $charid = strtoupper(md5(uniqid(mt_rand(), true)));
        $hyphen = chr(45); // "-"
        $uuid = substr($charid, 0, 8) . $hyphen
            . substr($charid, 8, 4) . $hyphen
            . substr($charid, 12, 4) . $hyphen
            . substr($charid, 16, 4) . $hyphen
            . substr($charid, 20, 12);
        return $uuid;
    }

    /**
     * Process the payment
     *
     * @param int  $order_id Reference.
     * @param bool $retry Should we retry on fail.
     * @param bool $force_customer Force user creation.
     *
     * @throws Exception If payment will not be accepted.
     *
     * @return array|void
     */
    public function process_payment($order_id, $retry = true, $force_customer = false) {
        //$rss = var_export($_REQUEST, TRUE);
        //throw new Exception($rss);
        if (!$_REQUEST['ipasspay-direct-card-number']) {
            throw new Exception("Credit Card number cannot be NULL");
        }

        if (!$_REQUEST['ipasspay-direct-card-expiry']) {
            throw new Exception("Expiry date cannot be NULL");
        }

        if (!$_REQUEST['ipasspay-direct-card-cvc']) {
            throw new Exception("CVV cannot be NULL");
        }

        //过期年月信息
        $ex_my = explode("/", $_REQUEST['ipasspay-direct-card-expiry']);
        $ex_month = $this->trimall($ex_my[0]);
        $ex_year = $this->trimall(substr($ex_my[1], -2));

        if (!$ex_month || !$ex_year) {
            throw new Exception("01|Invalid expiry date");
        }

        //第一步验证信用卡信息
        include_once dirname(__FILE__) . '/class-wc-ccvalidation.php';

        $cc_valid = new cc_validation();
        $cc_result = $cc_valid->validate($this->trimall($_REQUEST['ipasspay-direct-card-number']), $ex_month, $ex_year);
        //$cc_result = $cc_valid->validate($this->trimall($_REQUEST['ipasspay-direct-card-number']), "01", "19");
        $error = "";
        switch ($cc_result) {
            case 1:
                $error = "";
                break;
            case -1:
                $error = "01|Invalid card number";
                break;
            case -2:
            case -3:
            case -4:
                $error = "02|Invalid expiry date";
                break;
            case false:
                $error = "02|Invalid card number";
                break;
        }

        if ($error) {
            throw new Exception($error);
        }

        //引入支付KIT
        //include_once dirname(__FILE__) . '/class-wc-ipasspay-direct-api.php';
        $order = wc_get_order($order_id);
        if ($order->get_total() * 100 < 50) {
            throw new Exception(__('Sorry, the minimum allowed order total is 0.50 to use this payment method.', 'woocommerce-gateway-ipasspay-direct'));
        }

        //$ipasspay-direct_oop = new Directpaykit($this->mid, $this->app_id, $this->api_secret,$this->gateway_url );
        $post_data['currency'] = strtolower(version_compare(WC_VERSION, '3.0.0', '<') ? $order->get_order_currency() : $order->get_currency());
        $amount = number_format($this->get_ipasspay_direct_amount($order->get_total(), $post_data['currency']), 2, ".", "");

        //var_dump($order);exit;
        $curlPost['merchant_id'] = $this->mid;
        $curlPost['app_id'] = $this->app_id;
        $curlPost['order_no'] = time() . rand(111111,999999)."wordpress".$order_id;
        $curlPost['order_currency'] = strtoupper($post_data['currency']); //ex:USD,CNY,EUR,GBP
        $curlPost['order_amount'] = $amount;
        $curlPost['order_items'] = 'goods';
        $curlPost['card_no'] = $this->trimall($_REQUEST['ipasspay-direct-card-number']); //card number
        $curlPost['card_ex_year'] = $ex_year;
        $curlPost['card_ex_month'] = $ex_month;
        $curlPost['card_cvv'] = $this->trimall($_REQUEST['ipasspay-direct-card-cvc']); //cvv/cvc/cvv2
        $curlPost['source_ip'] = ($this->getIP()=='::1')?'127.0.0.1':$this->getIP();
        $curlPost['source_url'] = $this->curPageURL();
        $curlPost['asyn_notify_url'] = ($this->asyn_url == '')?($this->curPageURL() . '/wc-api/wc_gateway_ipasspay_direct'):$this->asyn_url;
        $curlPost['syn_notify_url'] = $this->curPageURL();
        $curlPost['version'] = $this->gateway_version;
        $curlPost['signature'] = hash('sha256', $curlPost['merchant_id'] . $curlPost['app_id']  . $curlPost['order_no'] . $curlPost['order_amount'] . $curlPost['order_currency'] . $this->api_secret);

        $curlPost['bill_firstname'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->billing_first_name : $order->get_billing_first_name();
        $curlPost['bill_lastname'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->billing_last_name : $order->get_billing_last_name();
        $curlPost['bill_email'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->billing_email : $order->get_billing_email();
        $curlPost['bill_phone'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->billing_phone : $order->get_billing_phone();
        $curlPost['bill_country'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->billing_country : $order->get_billing_country();
        $curlPost['bill_state'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->billing_state : $order->get_billing_state();
        $curlPost['bill_state'] = $curlPost['bill_state']?$curlPost['bill_state']:"--";
        $curlPost['bill_city'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->billing_city : $order->get_billing_city();
        $curlPost['bill_street'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->billing_address_1 : $order->get_billing_address_1();
        $curlPost['bill_zip'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->billing_postcode : $order->get_billing_postcode();

        $curlPost['ship_firstname'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->shipping_first_name : $order->get_shipping_first_name();
        $curlPost['ship_lastname'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->shipping_last_name : $order->get_shipping_last_name();
        $curlPost['ship_email'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->billing_email : $order->get_billing_email();
        $curlPost['ship_phone'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->billing_phone : $order->get_billing_phone();
        $curlPost['ship_country'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->shipping_country : $order->get_shipping_country();
        $curlPost['ship_state'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->shipping_state : $order->get_shipping_state();
        $curlPost['ship_state'] = $curlPost['ship_state']?$curlPost['ship_state']:"--";
        $curlPost['ship_city'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->shipping_city : $order->get_shipping_city();
        $curlPost['ship_street'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->shipping_address_1 : $order->get_shipping_address_1();
        $curlPost['ship_zip'] = version_compare(WC_VERSION, '3.0.0', '<') ? $order->shipping_postcode : $order->get_shipping_postcode();
        $curlPost['custom_data'] = $order_id;
        $curlPost['lang'] = 'en-us'; //zh-cn en-us
        //var_dump($curlPost);exit;
        $response = $this->curlSend($this->gateway_url,$curlPost,1);

        if (!$response) {
            throw new Exception("invalid response,please try again...");
        }

        $result = json_decode($response,true);

        if($result["data"]["pay_url"] != '') {
            //3D channel payment
            $redirect_url = urldecode($result["data"]["pay_url"]);
            return array(
                'result' => 'success',
                'redirect' => $redirect_url,
            );
        } else {  // 2D channel payment
            if ($result['errcode'] != 0) {
                throw new Exception($result['errmsg']);
            } else {
                switch ($result['data']['order_status']) {
                    case 2://成功
                    case 4://已授权
                        $order->payment_complete($result['data']['custom_data']);
                        $message = sprintf(__('Ipasspay charge complete (Charge ID: %s)', 'woocommerce-gateway-ipasspay'), $result['data']['custom_data']);
                        $order->add_order_note($message);
                        //$this->log('Success: ' . $message);
                        // Remove cart.
                        WC()->cart->empty_cart();
                        // Return thank you page redirect.
                        return array(
                            'result' => 'success',
                            'redirect' => $this->get_return_url($order),
                        );
                        break;
                    case 3: //失败
                        throw new Exception($result['data']['info']);
                        break;
                    default: //未知
                        throw new Exception($result['data']['info']);
                        break;
                }
            }
        }

    }

    /**
     * Sends the failed order email to admin
     *
     * @version 3.1.0
     * @since 3.1.0
     * @param int $order_id
     * @return null
     */
    public function send_failed_order_email($order_id) {
        $emails = WC()->mailer()->get_emails();
        if (!empty($emails) && !empty($order_id)) {
            $emails['WC_Email_Failed_Order']->trigger($order_id);
        }
    }

    public function curPageURL()
    {
        $pageURL = 'http';

        if ($_SERVER["HTTPS"] == "on")
        {
            $pageURL .= "s";
        }
        $pageURL .= "://";

        if ($_SERVER["SERVER_PORT"] != "80")
        {
            $pageURL .= $_SERVER["SERVER_NAME"] . ":" . $_SERVER["SERVER_PORT"];
        }
        else
        {
            $pageURL .= $_SERVER["SERVER_NAME"];
        }
        return $pageURL;
    }

    /**
     * 私有方法，CURL模拟提交
     * @param $process_gateway string 提交的网关地址
     * @param $data_arr 提交的数据数组
     * @param $type  模式提交的方式 1=》POST 0=》GET
     */

    public function getIP()
    {
        static $realip;
        if (isset($_SERVER)){
            if (isset($_SERVER["HTTP_X_FORWARDED_FOR"])){
                $realip = $_SERVER["HTTP_X_FORWARDED_FOR"];
            } else if (isset($_SERVER["HTTP_CLIENT_IP"])) {
                $realip = $_SERVER["HTTP_CLIENT_IP"];
            } else {
                $realip = $_SERVER["REMOTE_ADDR"];
            }
        } else {
            if (getenv("HTTP_X_FORWARDED_FOR")){
                $realip = getenv("HTTP_X_FORWARDED_FOR");
            } else if (getenv("HTTP_CLIENT_IP")) {
                $realip = getenv("HTTP_CLIENT_IP");
            } else {
                $realip = getenv("REMOTE_ADDR");
            }
        }
        return $realip;
    }

    public function notify()
    {
        $merchant_id = $this->mid;
        $app_id = $this->app_id;
        $api_secret = $this->api_secret;

        $order_id = $_REQUEST['custom_data'];
        $order_no = $_REQUEST['order_no'];
        $gateway_order_no = $_REQUEST['gateway_order_no'];
        $order_currency = $_REQUEST['order_currency'];
        $order_amount = $_REQUEST['order_amount'];
        $order_status = $_REQUEST['order_status'];
        $order_signature = $_REQUEST['signature'];
        $order = wc_get_order($order_id);

        $signature = hash('sha256', $merchant_id.$app_id.$order_no.
            $gateway_order_no.$order_currency.$order_amount.
            $order_status.$api_secret);

        //验签
        if ($signature == $order_signature) {
            switch ($order_status) {
                case '2':
                    $order->payment_complete();//状态改为processing，当管理员发货后，自行设置为complete
                    break;
                case '3':
                    $order->update_status('failed');
                    //$order->update_status( 'completed' );
                    break;
                default:
                    break;
            }
            $res = array(
                'errcode' => '0',
                'errmsg' => 'success'
            );
        } else {
            $res = array(
                'errcode' => '1',
                'errmsg' => 'failure'
            );
        }
        echo json_encode($res);
        exit;
    }

    private function curlSend($process_gateway, $data_arr, $type = 1) {
        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, $process_gateway);
        curl_setopt($ch, CURLOPT_POST, $type);
        curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($data_arr));
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, false);
        $response = curl_exec($ch);
        curl_close($ch);
        return $response;
    }



}

